<?php

namespace App\Http\Controllers;

use App\Models\Advertisement;
use App\Models\BusinessSettings;
use App\Models\Category;
use App\Models\Comment;
use App\Models\Game;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Models\GameReaction;
use App\Models\Languages;
use App\Models\Reply;
use App\Models\UserGameHistory;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Storage;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Http;

class GameController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index($slug) {
        $game = Game::where('slug', $slug)->where('status', 1)->firstOrFail();
        // Retrieve related games based on the game's category
        $relatedGames = Game::where('category_id', $game->category_id)->where('id', '!=', $game->id)->where('status', 1)->orderBy('id', 'DESC')->get();
        $popularGames = Game::where('status', 1)->orderBy('views', 'desc')->limit(36)->get();
        // Pass both the game and related games to the view
        $recentGames = Game::where('status', 1)->orderBy('id', 'DESC')->limit(30)->get();
        $categories = Category::where('is_active', 1)->orderBy('id', 'DESC')->get();
        $mostLikedGames = Game::withCount(['reactions as likes_count' => function ($query) {
            $query->where('like', true);
        }])->orderBy('likes_count', 'desc')->take(21)->get();
        $sidebar_ads = Advertisement::where('status', 'active')->where('position', 'sidebar')->get();
        $top_ads = Advertisement::where('status', 'active')->where('position', 'detail_page_top')->first();
        $bottom_ads = Advertisement::where('status', 'active')->where('position', 'detail_page_bottom')->first();
        $settings = BusinessSettings::pluck('value', 'key')->toArray();
        $hivecategories = Category::where('is_active', 1)->get();
        // Game Hive Theme recent games
        return view('frontend.details.index', compact('game', 'relatedGames', 'popularGames', 'recentGames', 'categories', 'mostLikedGames', 'sidebar_ads', 'top_ads', 'bottom_ads', 'settings', 'hivecategories'));
    }


    public function play($slug)
    {
        $game = Game::where('slug', $slug)->firstOrFail();

        if (Auth::check()) {
            // Check if the user has already visited the game
            $existingHistory = UserGameHistory::where('user_id', Auth::user()->id)->where('game_id', $game->id)->first();

            if ($existingHistory) {
                // Update the existing history entry
                $existingHistory->update([
                    'visited_at' => now(),
                ]);
            } else {
                // Create a new history entry
                UserGameHistory::create([
                    'user_id' => Auth::user()->id,
                    'game_id' => $game->id,
                    'visited_at' => now(),
                ]);
            }
        }

        // Increment game views
        $game->increment('views');

        // Get related and recent games
        $recentGames = Game::where('status', 1)->orderBy('id', 'DESC')->limit(30)->get();
        $relatedGames = Game::where('category_id', $game->category_id)
                            ->where('id', '!=', $game->id)
                            ->where('status', 1)
                            ->orderBy('id', 'DESC')
                            ->limit(21)
                            ->get();

        // Get game comments
        $comments = Game::with('comments.user')->findOrFail($game->id);

        // Generate the proxy URL to fetch and clean the game content
        $proxyUrl = route('proxy.game', ['slug' => $game->slug]);

        $sidebar_ads = Advertisement::where('status', 'active')->where('position', 'sidebar')->get();
        $settings = BusinessSettings::pluck('value', 'key')->toArray();
        $categories = Category::where('is_active', 1)->get();
        $gamegaloreRelated = Game::where('category_id', $game->category_id)
                                ->where('id', '!=', $game->id)
                                ->where('status', 1)
                                ->orderBy('id', 'DESC')
                                ->get();
        // Return the view with the proxy URL embedded in the iframe
        return view('frontend.play.index', compact('game', 'recentGames', 'relatedGames', 'comments', 'proxyUrl', 'sidebar_ads', 'settings', 'categories', 'gamegaloreRelated'));
    }

    public function storeComment(Request $request, $gameId)
    {
        $comment = new Comment();
        $comment->game_id = $gameId;
        $comment->user_id = Auth::id();
        $comment->comment = $request->comment;
        $comment->save();

        // Return JSON response for AJAX
        return response()->json([
            'user_name' => $comment->user->name,
            'user_profile_photo' => asset('images/user/'.$comment->user->image) ?? 'https://via.placeholder.com/40',
            'comment' => $comment->comment,
            'created_at' => $comment->created_at->diffForHumans(),
        ]);
    }

    public function storeReply(Request $request, $commentId)
    {
        // Validate the input
        $request->validate([
            'reply' => 'required|string|max:500',
        ]);

        // Check if the comment exists
        $comment = Comment::findOrFail($commentId);

        // Create the reply
        $reply = new Reply();
        $reply->comment_id = $comment->id; // Ensure the correct comment ID is used
        $reply->user_id = Auth::id();
        $reply->comment = $request->input('reply');
        $reply->save();

        // Return JSON response for AJAX
        return response()->json([
            'user_name' => Auth::user()->name,
            'reply' => $reply->comment,
            'created_at' => $reply->created_at->diffForHumans(),
        ]);
    }



    public function category($slug) {
        $category = Category::where('slug', $slug)->firstOrFail();
        $games = Game::where('category_id', $category->id)->where('status', 1)->orderBy('id', 'DESC')->get();
        $categories = Category::where('is_active', 1)->orderBy('id', 'DESC')->get();
        $settings = BusinessSettings::pluck('value', 'key')->toArray();
        $hivecategories = Category::where('is_active', 1)->get();
        return view('frontend.categories.index', compact('games', 'category', 'categories', 'settings', 'hivecategories'));
    }

    public function search(Request $request)
    {
        $query = $request->input('search');

        $games = Game::where(function ($q) use ($query) {
            $q->where('name', 'like', '%' . $query . '%')
            ->orWhere('description', 'like', '%' . $query . '%')
            ->orWhere('tags', 'like', '%' . $query . '%')
            ->orWhereHas('category', function ($q) use ($query) {
                $q->where('name', 'like', '%' . $query . '%');
            });
        })
        ->where('status', 1)
        ->orderBy('id', 'DESC')
        ->get();

        $categories = Category::where('is_active', 1)->orderBy('id', 'DESC')->get();
        $settings = BusinessSettings::pluck('value', 'key')->toArray();
        $hivecategories = Category::where('is_active', 1)->get();

        return view('frontend.search.index', compact('games', 'query', 'categories', 'settings', 'hivecategories'));
    }

    // Admin Routes
    public function games(){
        $games = Game::orderBy('id', 'DESC')->paginate(15);
        return view('backend.games.index', compact('games'));
    }

    public function FeaturedGames(){
        $games = Game::where('featured', 1)->orderBy('id', 'desc')->paginate(15);
        return view('backend.games.featured-games', compact('games'));
    }

    public function DisabledGames(){
        $games = Game::where('status', 0)->orderBy('id', 'desc')->paginate(15);
        return view('backend.games.disabled-games', compact('games'));
    }

    public function create(){
        $categories = Category::where('is_active', 1)->orderBy('id', 'DESC')->get();
        return view('backend.games.create', compact('categories'));
    }

    public function store(Request $request) {
        // Validate the incoming request data
        $request->validate([
            'name' => 'required|string|max:255',
        ]);

        // Handle image upload for the game icon
        if ($request->hasFile('image')) {
            $imageName = 'icon' . time() . '.' . $request->image->extension();
            $request->image->move(public_path('images/games/icons'), $imageName);

        }

        // Handle banner upload for the game (if provided)
        if ($request->hasFile('banner')) {
            $bannerName = 'banner' . time() . '.' . $request->banner->extension();
            $request->banner->move(public_path('images/games/banner'), $bannerName);
        }

        // Create a new Game record
        $game = new Game();
        $game->name = $request->input('name');
        $game->slug = Str::slug($request->input('name') . time());
        $game->category_id = $request->input('category_id');
        $game->description = $request->input('description');
        $game->game_url = $request->input('game_url');
        $game->meta_title = $request->input('meta_title');
        $game->meta_description = $request->input('meta_description');
        $game->image = $imageName;
        $game->banner = $bannerName;
        $game->tags = $request->input('tags');
        $game->featured = $request->input('featured');
        $game->status = $request->input('status');

        // Save the game to the database
        $game->save();

        // Redirect to a specific route with a success message
        return redirect()->route('admin.games')->with('success', 'Game added successfully!');
    }

    public function edit($id){
        $game = Game::findOrFail($id);
        $categories = Category::where('is_active', 1)->orderBy('id', 'DESC')->get();
        $languages = Languages::where('status', 1)->get();

        // Fetch translations by language code
        $translations = $game->translations->keyBy('locale'); // Key translations by locale instead of language_code
        return view('backend.games.edit', compact('game', 'categories', 'languages', 'translations'));
    }

    public function update(Request $request) {
        if (env('DEMO_MODE') == 'on') {
            return redirect()->back()->with('error', 'This action is disabled in demo mode.');
        } else {
            $game = Game::findOrFail($request->id);

            // Handle image upload for the game icon
            if ($request->hasFile('image')) {
                $imageName = 'icon' . time() . '.' . $request->image->extension();
                $request->image->move(public_path('images/games/icons'), $imageName);
                $game->image = $imageName;
            }

            // Handle banner upload for the game (if provided)
            if ($request->hasFile('banner')) {
                $bannerName = 'banner' . time() . '.' . $request->banner->extension();
                $request->banner->move(public_path('images/games/banner'), $bannerName);
                $game->banner = $bannerName;
            }

            $game->name = $request->input('name');
            $game->category_id = $request->input('category_id');
            $game->description = $request->input('description');
            $game->game_url = $request->input('game_url');
            $game->meta_title = $request->input('meta_title');
            $game->meta_description = $request->input('meta_description');
            $game->tags = $request->input('tags');
            $game->featured = $request->input('featured');

            // Save the game to the database
            $game->save();

            if ($request->has('translations')) {
                foreach ($request->translations as $languageCode => $translationData) {
                    // Only save translation if name or description is provided
                    if (!empty($translationData['name']) || !empty($translationData['description'])) {
                        $game->translations()->updateOrCreate(
                            ['locale' => $languageCode, 'game_id' => $game->id],  // Add game_id explicitly
                            [
                                'name' => $translationData['name'] ?? '',
                                'description' => $translationData['description'] ?? '',
                            ]
                        );
                    }
                }
            }

            // Redirect to a specific route with a success message
            return redirect()->route('admin.games')->with('success', 'Game updated successfully!');
        }
    }

    public function delete($id){
        // Delete with unlink images


        if (env('DEMO_MODE') == 'on') {
            return redirect()->back()->with('error', 'This action is disabled in demo mode.');
        } else {
            $game = Game::findOrFail($id);
            if ($game->image) {
                $imagePath = public_path('images/games/icons/' . $game->image);
                if (file_exists($imagePath)) {
                    unlink($imagePath);
                }
            }
            if ($game->banner) {
                $bannerPath = public_path('images/games/banner/' . $game->banner);
                if (file_exists($bannerPath)) {
                    unlink($bannerPath);
                }
            }
            $game->delete();
            return redirect()->route('admin.games')->with('success', 'Game deleted successfully!');
        }

    }

    public function likeGame($id)
    {
        $game = Game::findOrFail($id);
        $reaction = GameReaction::firstOrNew([
            'user_id' => Auth::id(),
            'game_id' => $id,
        ]);

        if ($reaction->like) {
            // If the user has already liked, remove the like
            $reaction->like = false;
        } else {
            // If the user has disliked or not reacted, set like to true
            $reaction->like = true;
            $reaction->dislike = false; // Remove dislike if it was set
        }
        $reaction->save();

        $likes = $game->reactions->where('like', true)->count();
        $dislikes = $game->reactions->where('dislike', true)->count();

        return response()->json(['likes' => $likes, 'dislikes' => $dislikes]);
    }

    public function dislikeGame($id)
    {
        $game = Game::findOrFail($id);
        $reaction = GameReaction::firstOrNew([
            'user_id' => Auth::id(),
            'game_id' => $id,
        ]);

        if ($reaction->dislike) {
            // If the user has already disliked, remove the dislike
            $reaction->dislike = false;
        } else {
            // If the user has liked or not reacted, set dislike to true
            $reaction->dislike = true;
            $reaction->like = false; // Remove like if it was set
        }
        $reaction->save();

        $likes = $game->reactions->where('like', true)->count();
        $dislikes = $game->reactions->where('dislike', true)->count();

        return response()->json(['likes' => $likes, 'dislikes' => $dislikes]);
    }

    public function fetchGames(Request $request)
    {
        // Retrieve form inputs
        $numGames = $request->input('num_games');
        $gameOrder = $request->input('game_order'); // 'pubdate' or default
        $gamepixsid = BusinessSettings::where('key', 'gamepix_sid')->first()->value;

        // Base API URL
        $apiUrl = 'https://feeds.gamepix.com/v2/json?sid='.$gamepixsid.'&pagination=' . $numGames . '&page=1';

        // Add order parameter if needed
        if (!empty($gameOrder)) {
            $apiUrl .= '&order=' . $gameOrder;
        }

        // Make API request to fetch the games
        $response = Http::get($apiUrl);

        // Check if the API request was successful
        if ($response->successful()) {
            // Parse the games from the response
            $games = $response->json()['items'];

            // Return JSON response for AJAX
            return response()->json(['games' => $games]);
        } else {
            // Handle the case where the API request fails
            return response()->json(['error' => 'Failed to fetch games'], 500);
        }
    }

    public function addGame(Request $request)
    {
        // Validate request data
        $request->validate([
            'title' => 'required|string|max:255',
            'slug' => 'required|string|max:255',
            'category' => 'required|string|max:255',
            'image' => 'required|url',
            'banner' => 'required|url',
            'url' => 'required|url',
            'description' => 'required|string',
        ]);

        // Define a list of category mappings for normalization
        $categoryMappings = [
            'hypercasual' => 'hyper-casual',
            'hyper-casual' => 'hyper-casual',
            'car race' => 'car racing',
            'car racing' => 'car racing',
            'shooter' => 'shooting',
            'shooting' => 'shooting',
            'games-for-girls' => 'Girls',
            'girls' => 'Girls',
            'match-3' => 'strategy',
            'strategy' => 'strategy',
        ];

        // Get the requested category and normalize it
        $requestedCategory = Str::slug($request->category);
        $normalizedCategory = $categoryMappings[$requestedCategory] ?? $request->category;

        // Check if the category exists or create a new one
        $category = Category::firstOrCreate(
            ['slug' => Str::slug($normalizedCategory)],
            ['name' => $normalizedCategory]
        );

        // Check if a game with the same slug or name already exists
        $existingGame = Game::where('slug', Str::slug($request->slug))
                            ->orWhere('name', $request->title)
                            ->first();

        if ($existingGame) {
            return response()->json([
                'success' => false,
                'message' => 'A game with the same name or slug already exists.',
            ], 422);
        }

        // Download images for the game
        $imagePath = $this->downloadImage($request->image, 'icons');
        $bannerPath = $this->downloadImage($request->banner, 'banner');

        // Create a new game record
        $game = Game::create([
            'category_id'      => $category->id,
            'name'             => $request->title,
            'description'      => $request->description,
            'image'            => $imagePath,
            'banner'           => $bannerPath,
            'slug'             => Str::slug($request->slug),
            'game_url'         => $request->url,
            'meta_title'       => $request->title,
            'meta_description' => $request->description,
            'tags'             => $request->tags,
        ]);

        return response()->json(['success' => true, 'message' => 'Game added successfully.']);
    }


    // Helper function to download and save the image
    private function downloadImage($url, $folder)
    {
        $cleanUrl = strtok($url, '?'); // Remove query parameters

        // Get the image extension and name
        $fileExtension = pathinfo($cleanUrl, PATHINFO_EXTENSION);
        $imageName = time() . '.' . $fileExtension;

        // Define the image path where it will be stored
        $imagePath = public_path('images/games/' . $folder . '/' . $imageName);

        // Create the folder if it doesn't exist
        if (!file_exists(public_path('images/games/' . $folder))) {
            mkdir(public_path('images/games/' . $folder), 0775, true);
        }

        // Use cURL to download the image
        $ch = curl_init($cleanUrl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); // Follow redirects
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Skip SSL verification (optional)

        $imageData = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if ($httpCode == 200 && $imageData) {
            file_put_contents($imagePath, $imageData);
        } else {
            return null; // Handle error case
        }

        curl_close($ch);

        return $imageName; // Return relative path
    }


    public function proxyGame($slug)
    {
        $client = new Client();

        // Fetch the game based on slug
        $game = Game::where('slug', $slug)->firstOrFail();

        // URL to fetch the external game content
        $gameUrl = $game->game_url; // Make sure you have 'game_url' field in the Game model

        try {
            // Send a GET request to the external game URL
            $response = $client->request('GET', $gameUrl);

            // Get the HTML content of the game
            $htmlContent = $response->getBody()->getContents();

            // Load the HTML into DOMDocument
            $dom = new \DOMDocument();
            @$dom->loadHTML($htmlContent);

            // Use XPath to find and remove ad elements
            $xpath = new \DOMXPath($dom);

            // Expanded list of common ad-related classes and IDs, including Zemanta and Criteo
            $adClassesAndIds = [
                'ad', 'ads', 'advertisement', 'ad-container', 'adclick', 'ad-banner',
                'google-ads', 'adsbygoogle', 'adsense', 'sponsored', 'sponsored-link',
                'ad-unit', 'ad-slot', 'ad-wrapper', 'ad-space', 'promo', 'promoted',
                'ad-footer', 'ad-header', 'adbox', 'adslot', 'banner-ad', 'ads-widget',
                'advert', 'ad_area', 'ad-block', 'adframe', 'adspot', 'sponsor',
                'zemanta', 'zem-ad', 'zemanta-ad', 'zemanta-sponsored', 'native-ad',
                'criteo', 'criteo-banner', 'criteo-ad', 'criteo-sponsored', 'criteo-content', 'criteo-native'
            ];

            // Iterate through the list and remove ad elements
            foreach ($adClassesAndIds as $adClass) {
                foreach ($xpath->query("//*[contains(@class, '$adClass') or contains(@id, '$adClass')]") as $adElement) {
                    $adElement->parentNode->removeChild($adElement);
                }
            }

            // Return the cleaned game content to be loaded in the iframe
            return response($dom->saveHTML())->header('Content-Type', 'text/html');
        } catch (\Exception $e) {
            // Handle errors
            return response("Unable to load the game", 500);
        }
    }


    public function toggleStatus(Request $request){
        if (env('DEMO_MODE') == 'on') {
            return redirect()->back()->with('error', 'This action is disabled in demo mode.');
        } else {
            $game = Game::findOrFail($request->id);
            $game->status = $request->status;
            $game->save();

            return response()->json(['success' => true]);
        }
    }


}
